// ENCODING: UTF-8
'use strict';

// BUILT-IN MODULES
var fs = require('fs');

// EXTERNAL MODULES
var iniparser = require('./../iniparser');
var Long = require('./../long');

//################################################
// TotalCommander
//################################################
var m_nSeed = 0; // UINT = 0..4294967295=FFFFFFFF

function totalCommanderDecrypt(encrypted){
	Number.prototype.mod = function(n) {
		return ((this%n)+n)%n;
	}
	var i;
	var decrypted = '';
	var nLen = encrypted.length/2 - 4;
	if(nLen < 0) return false;
	var password = [];
	var nFirst = 0;
	var nLast = 0;
	var u = 0x0;

	for (i=0;i<nLen;i++)
		password.push( parseInt(encrypted.substr(i*2,2), 16));
	m_nSeed = 849521;
	for (i=0; i<nLen; i++)
		password[i] = roll(password[i], makeRandom(8));
	m_nSeed = 12345;
	for (i=0; i<256; i++){
		nFirst = makeRandom(nLen);
		nLast = makeRandom(nLen);		
		u = password[nFirst];
		password[nFirst] = password[nLast];
		password[nLast] = u;
	}
	m_nSeed = 42340;
	for (i=0; i<nLen; i++) {
		password[i] ^= makeRandom(256);
		password[i] = password[i].mod(0x100);
	}
	m_nSeed = 54321;
	for(i=0; i<nLen; i++){
		password[i] -= makeRandom(256);
		password[i] = password[i].mod(0x100);
	}
	//return password; // Array
	return String.fromCharCode.apply(null, password); // String
}

function makeRandom(nMax){
	// ORG C-CODE
	// m_nSeed = (m_nSeed * 0x8088405) + 1;
	// return (int)(((LONGLONG)m_nSeed * nMax) >> 32);
	
	m_nSeed = new Long(m_nSeed, 0, true);
	m_nSeed = m_nSeed.multiply( new Long(0x8088405, 0, true) );
	m_nSeed = m_nSeed.low.mod(0x100000000) + 1;
	var ret = new Long(m_nSeed, 0, true);
	ret = ret.multiply( new Long(nMax, 0, true) );
	ret = ret.shiftRight(32);
	return ret.toInt();
}

// UINT roll(UINT n1, UINT n2);
function roll(n1, n2){
	var res =((n1 << n2) | (n1 >> (8 - n2)));
	return res.mod(0x100000000);
}

var format = {};
format['name'] = 'Total Commander';
format['spec'] = 'wcx_ftp.ini';
format['load'] = function(fn, cb){
	var data = iniparser.parseSync(fn);
	var names = data['connections'];
	if (names['default']) delete names['default'];
	//more properties:
	// - pasvmode=0
	var sites = {};
	for (var i in names){
		var name = names[i];
		var info = data[name];
		var site = {};
		var tmp = info['host'].split(':');
		site.host = tmp[0];
		site.port = tmp.length>1?tmp[1]*1:21;
		if (info['username']){
			site.user = info['username'];
			site.pass = totalCommanderDecrypt(info['password']);
		}
		if (info['directory']) site.rdir = info['directory'];
		sites[name] = site;
	}
	cb(sites);
}

module.exports = format;