// ENCODING: UTF-8
'use strict';

// BUILT-IN MODULES
var fs = require('fs');

// EXTERNAL MODULES
var iniparser = require('./../iniparser');

//################################################
// WS_FTP
// source: http://lab.artlung.com/ws-ftp-password-decoder/
//################################################
function wsFtpDecrypt(in_string) {
	if (in_string.indexOf('PWD=', 0) === -1 || in_string.length < 37) {
		//throw new Error("ENTRY NOT VALID: be sure to enter the whole line, including 'PWD='");
		return false;
	}
	var my_password = in_string.substring(37, in_string.length);
	var decoded = '';
	for (var i=0; i<my_password.length/2; i++) {
		var my_character = my_password.substring(i*2, i*2+2);
		var my_parsed = in_string.substring(5+i, 6+i);
		var my_clear_txt = parseInt('0x'+my_character)-i-1-((47 + parseInt('0x'+my_parsed)) % 57);
		decoded += String.fromCharCode(my_clear_txt);
	}
	return decoded;
}

var format = {};
format['name'] = 'WS_FTP';
format['spec'] = 'ws_ftp.ini';
format['load'] = function(fn, cb){
	var data = iniparser.parseSync(fn);
	if (data['_config_']) delete data['_config_'];
	for (var name in data){
		var info = data[name];
		var site = {};
		site.host = info['HOST'];
		site.port = info['PORT']?info['PORT']*1:21;
		site.user = info['UID'];
		if (info['PWD']){
			site.pass = wsFtpDecrypt('PWD='+info['PWD']);
		}
		if (info['DIR']) site.rdir = info['DIR'];
		//more properties:
		// - PASVMODE=1
		// - TIMEOFFSET=0
		// make unique name
		var name = info['Account Name'];
		sites[name] = site;
	}
	cb(sites);
}

module.exports = format;